// this script will be run within the webview itself thorugh the embedded web engine
// it is run as soon and the extension is activated


(function () {

	// initialization code, first we get the vscode api object then setup some variables for maintaining the webview state
	// then we attack some event listeners to communicate between extension code and webview code

	const vscode = acquireVsCodeApi();

	let variables = undefined;						// array that keeps a global list of variables found in the file.
	let is_showing = undefined;

	// handle messages sent from the extension to the webview
	window.addEventListener('message', event => {
		const message = event.data;					// the json data that the extension sent
		switch (message.command) {
			case 'post_data':						// the webview asks the extension to send it back data. list of variables usually
				variables = message.data;
				console.log(variables);
				addToSelector();
				break;
		}
	});

	// as soon as we are able, we ask the extension for a list of variables from the file.
	vscode.postMessage({ command: "get_data" });

	// attach some event listeners
	document.getElementById("btn_show").addEventListener("click", showVars);
	document.getElementById("btn_generate").addEventListener("click", insertIntoResult);
	document.getElementById("btn_insert").addEventListener("click", insertIntoDocument);
	document.getElementById("btn_or").addEventListener("click", or);
	document.getElementById("btn_and").addEventListener("click", and);
	document.getElementById("btn_not").addEventListener("click", not);
	document.getElementById("btn_diff").addEventListener("click", diff);
	document.getElementById("btn_like").addEventListener("click", like);
	document.getElementById("btn_class").addEventListener("click", insertClass);

	// define the simplistic grammar for syntax highlighting
	CodeMirror.defineSimpleMode("aql",
		{
			start: [
				{ regex: /"(?:[^\\]|\\.)*?(?:"|$)/, token: "string" },
				{ regex: /<.*?>/, token: "atom" },
				{ regex: /(\(|\)|\[|\]|\?)/, token: "keyword" },
				{ regex: /(AND|and|OR|or|NOT|not|DIFF|diff|LIKE|like|ILIKE|ilike)/, token: "operator" },
				{ regex: /\${.*?}\$/, token: "builtin" }
			]
		}
	);

	// start the code editor and keep the object globally
	var codeMirrorEditor = CodeMirror.fromTextArea(
		document.getElementById("text_query"),
		{
			mode: "aql",
			theme: "dracula",
			lineNumbers: true,
			autoCloseTags: true,
			autoCloseBrackets: "()[]{}\'\'\"\"<>",
			lineWrapping: true
		}
	);

	// next we define the functions attaached to the events and some other helper functions

	// shows the list of variables when you click the Show Variable button
	function showVars() {
		if (is_showing === undefined) {

			let div = document.getElementById("var_ctn");

			let list = document.createElement("ol");
			div.appendChild(list);

			for (let i = 0; i < variables.length; ++i) {
				let item = document.createElement("li");

				item.addEventListener("click", pasteIntoTextarea);

				let name_and_value = variables[i].name.split("<--!-->");

				item.innerHTML = name_and_value[0] + ": " + name_and_value[1];

				list.appendChild(item);
			}

			is_showing = true;

		} else if (is_showing === true) {
			document.getElementById("var_ctn").style.visibility = "hidden";
			is_showing = false;

		} else if (is_showing === false) {
			document.getElementById("var_ctn").style.visibility = "visible";
			is_showing = true;
		}
	}

	function addToSelector() {
		let datalist = document.getElementById("options_class");

		let first_option = document.createElement("option");
		first_option.innerHTML = "";

		datalist.appendChild(first_option);

		for (let i = 0; i < variables.length; ++i) {
			let option = document.createElement("option");
			
			let name_and_value = variables[i].name.split("<--!-->");

			option.value = name_and_value[0];
			option.innerHTML = name_and_value[0] + ": " + name_and_value[1];

			datalist.appendChild(option);
		}
	}

	// the big function that translates an AQL query into a string.
	// it escapes quotes, parethesis and also adds quotes, pluses and spaces
	function translate(og_query) {
		let characters = og_query.split("");

		let translation = "";

		for (let i = 0; i < characters.length; ++i) {
			let char = characters[i];

			if (char === "\"") {
				translation += "\\\""
			} else {
				translation += char;
			}
		}

		const regex = /\${(.+?)}\$/;

		while (regex.test(translation)) {
			let match = translation.match(regex);

			let splitted_first = translation.substring(0, match["index"]);
			let splitted_second = translation.substring(match["index"] + match[0].length);

			translation = splitted_first.concat("\\\"\" + " + match[1] + " + \"\\\"", splitted_second);
		}

		return "\"" + translation + "\"";
	}

	function insertIntoResult(event) {
		let str = translate(codeMirrorEditor.doc.getValue());

		document.getElementById("text_result").value = str;
	}

	function insertIntoDocument(event) {
		// first we want to translate into a well formed string and send it to the extension, and replace the selection with it
		let str = translate(codeMirrorEditor.doc.getValue());

		vscode.postMessage(
			{
				command: "send_string",
				string: str
			}
		);
	}

	function or(event) {
		let pos = codeMirrorEditor.doc.getCursor();

		codeMirrorEditor.doc.replaceRange(" OR ", pos);
	}

	function and(event) {
		let pos = codeMirrorEditor.doc.getCursor();

		codeMirrorEditor.doc.replaceRange(" AND ", pos);
	}

	function not(event) {
		let pos = codeMirrorEditor.doc.getCursor();

		codeMirrorEditor.doc.replaceRange(" NOT ", pos);
	}

	function diff(event) {
		let pos = codeMirrorEditor.doc.getCursor();

		codeMirrorEditor.doc.replaceRange(" DIFF ", pos);
	}

	function like(event) {
		let pos = codeMirrorEditor.doc.getCursor();

		codeMirrorEditor.doc.replaceRange(" LIKE ", pos);
	}

	function insertClass(event) {
		let var_name = document.getElementById("sel_class").value;

		if (var_name !== "") {
			let formatted = "<${" + var_name + "}$>";

			let pos = codeMirrorEditor.doc.getCursor();

			codeMirrorEditor.doc.replaceRange(formatted, pos);
		}
	}

	// helper function to paste variable names into the text editor
	function pasteIntoTextarea(event) {

		let var_name = event.target.innerHTML.split(": ")[0];

		let formatted = "${" + var_name + "}$";

		let pos = codeMirrorEditor.doc.getCursor();

		codeMirrorEditor.doc.replaceRange(formatted, pos);
	}
}());
