import * as vscode from 'vscode';

// here we define a symbol provider that uses vscode machinery to give us back a list of variables from the file
export default class AscDocumentSymbolProvider implements vscode.DocumentSymbolProvider {

	// methord declared by the interface. it returns a list of Document Symbols that then vscode uses
	public provideDocumentSymbols(document: vscode.TextDocument, token: vscode.CancellationToken):
		vscode.ProviderResult<vscode.DocumentSymbol[]> {
		const text = document.getText();

		let ret: vscode.DocumentSymbol[] = [];

		let variableRegex = /SET[LG]\s?([a-z][a-zA-Z_0-9\\-]*)\s?:\s?(\((.*)\)|(.*))/g;
		let variableRegex2 = /SET[LG]\s?([a-z][a-zA-Z_0-9\\-]*)\s?:\s?(\((.*)\)|(.*))/;

		// parse the whole file
		let matches = document.getText().match(variableRegex);

		if (matches !== null) {
			for (let i = 0; i < matches.length; ++i) {
				let range = new vscode.Range(document.positionAt(text.indexOf(matches[i])),
					document.positionAt(text.indexOf(matches[i]) + matches[i].length));
				let name_and_value = document.getText(range);
				let detail = "";
				let kind = vscode.SymbolKind.Variable;

				let match = name_and_value.match(variableRegex2);
				let name;
				let value;

				if (match !== null) {
					name = match[1];
					value = match[3];
				}

				let doc_symb = new vscode.DocumentSymbol(name + "<--!-->" + value, detail, kind, range, range);

				ret.push(doc_symb);
			}
		}

		return ret;
	}
}
