import * as vscode from 'vscode';

// we import the classes that provide symbol parsing and the class that represents our webview
import ascDocumentSymbolProvider from './AscDocumentSymbolProvider';
import InterfacePanel from './InterfacePanel';

// we keep track of the last opened text editor for when we need to insert text to it
let last_active_window: vscode.TextEditor | undefined;

// the activate function is ran when vscode executes one of the activation events we subscribe to in the package.json file
export function activate(context: vscode.ExtensionContext): void {

	context.subscriptions.push(
		vscode.commands.registerCommand('adoxx-aql-queryEditor.start', () => {
			// before we create/show the webview panel we gather a list of variables from the active file.
			// for that we get the whole document text, loop through line by line checking a regular expression.
			last_active_window = vscode.window.activeTextEditor;

			if (last_active_window?.selection) {
				let selection = last_active_window.selection;
				let text: string = last_active_window.document.getText(new vscode.Range(selection.start, selection.end));

				InterfacePanel.createOrShow(context.extensionPath, text);
			} else {
				InterfacePanel.createOrShow(context.extensionPath);
			}
		})
	);

	context.subscriptions.push(
		vscode.languages.registerDocumentSymbolProvider('adoxx-ado-script', new ascDocumentSymbolProvider())
	);

	context.subscriptions.push(
		vscode.commands.registerCommand('adoxx-adoscript.runSymbolProvider', runSymbolProvider)
	);
}

// helper function that we attach to a command, that runs the executeDocumentSymbolProvider command
export async function runSymbolProvider() {
	let ret;

	if (last_active_window !== undefined) {
		ret = await vscode.commands.executeCommand<vscode.DocumentSymbol[]>(
			"vscode.executeDocumentSymbolProvider", last_active_window.document.uri
		);
	}

	return ret;
}

// replace selected text with the str
export function insertString(str: string): void {
	if (last_active_window !== undefined) {
		last_active_window.edit((edit) => {
			if (last_active_window !== undefined) {
				edit.replace(last_active_window.selection, str);
			}
		});
	}
}

// function that disposes of the webview panel when the extension is deactivated
export function deactivate(): void {
	InterfacePanel.currentPanel?.dispose();
}
